@@ -19,6 +19,9 @@ package org.apache.maven.plugin;
  * under the License.
  */
 
+import org.apache.maven.execution.MavenSession;
+import org.apache.maven.toolchain.Toolchain;
+import org.apache.maven.toolchain.ToolchainManager;
 import org.codehaus.plexus.compiler.Compiler;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
 import org.codehaus.plexus.compiler.CompilerError;
@@ -31,12 +34,13 @@ import org.codehaus.plexus.compiler.util.scan.SourceInclusionScanner;
 import org.codehaus.plexus.compiler.util.scan.mapping.SingleTargetSourceMapping;
 import org.codehaus.plexus.compiler.util.scan.mapping.SourceMapping;
 import org.codehaus.plexus.compiler.util.scan.mapping.SuffixMapping;
+import org.codehaus.plexus.util.ReaderFactory;
 import org.codehaus.plexus.util.StringUtils;
 
 import java.io.File;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -50,7 +54,8 @@ import java.util.Set;
  *
  * @author others
  * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
- * @version $Id: StaleSourceScannerTest.java 2393 2005-08-08 22:32:59Z kenney $
+ * @version $Id: AbstractCompilerMojo.java 1336851 2012-05-10 19:39:09Z hboutemy $
+ * @since 2.0
  */
 public abstract class AbstractCompilerMojo
     extends AbstractMojo
@@ -60,21 +65,22 @@ public abstract class AbstractCompilerMojo
     // ----------------------------------------------------------------------
 
     /**
-     * Indicates whether the build will continue even if there are compilation errors; defaults to true.
+     * Indicates whether the build will continue even if there are compilation errors.
      *
      * @parameter expression="${maven.compiler.failOnError}" default-value="true"
+     * @since 2.0.2
      */
     private boolean failOnError = true;
 
     /**
-     * Set to true to include debugging information in the compiled class files.
+     * Set to <code>true</code> to include debugging information in the compiled class files.
      *
      * @parameter expression="${maven.compiler.debug}" default-value="true"
      */
     private boolean debug = true;
 
     /**
-     * Set to true to show messages about what the compiler is doing.
+     * Set to <code>true</code> to show messages about what the compiler is doing.
      *
      * @parameter expression="${maven.compiler.verbose}" default-value="false"
      */
@@ -88,14 +94,14 @@ public abstract class AbstractCompilerMojo
     private boolean showDeprecation;
 
     /**
-     * Set to true to optimize the compiled code using the compiler's optimization methods.
+     * Set to <code>true</code> to optimize the compiled code using the compiler's optimization methods.
      *
      * @parameter expression="${maven.compiler.optimize}" default-value="false"
      */
     private boolean optimize;
 
     /**
-     * Set to true to show compilation warnings.
+     * Set to <code>true</code> to show compilation warnings.
      *
      * @parameter expression="${maven.compiler.showWarnings}" default-value="false"
      */
@@ -104,21 +110,22 @@ public abstract class AbstractCompilerMojo
     /**
      * The -source argument for the Java compiler.
      *
-     * @parameter expression="${maven.compiler.source}"
+     * @parameter expression="${maven.compiler.source}" default-value="1.5"
      */
-    private String source;
+    protected String source;
 
     /**
      * The -target argument for the Java compiler.
      *
-     * @parameter expression="${maven.compiler.target}"
+     * @parameter expression="${maven.compiler.target}" default-value="1.5"
      */
-    private String target;
+    protected String target;
 
     /**
      * The -encoding argument for the Java compiler.
      *
-     * @parameter expression="${maven.compiler.encoding}"
+     * @parameter expression="${encoding}" default-value="${project.build.sourceEncoding}"
+     * @since 2.1
      */
     private String encoding;
 
@@ -139,7 +146,7 @@ public abstract class AbstractCompilerMojo
     private String compilerId;
 
     /**
-     * Version of the compiler to use, ex. "1.3", "1.5", if fork is set to true.
+     * Version of the compiler to use, ex. "1.3", "1.5", if {@link #fork} is set to <code>true</code>.
      *
      * @parameter expression="${maven.compiler.compilerVersion}"
      */
@@ -147,30 +154,32 @@ public abstract class AbstractCompilerMojo
 
     /**
      * Allows running the compiler in a separate process.
-     * If "false" it uses the built in compiler, while if "true" it will use an executable.
+     * If <code>false</code> it uses the built in compiler, while if <code>true</code> it will use an executable.
      *
-     * @parameter default-value="false"
+     * @parameter expression="${maven.compiler.fork}" default-value="false"
      */
     private boolean fork;
 
     /**
      * Initial size, in megabytes, of the memory allocation pool, ex. "64", "64m"
-     * if fork is set to true.
+     * if {@link #fork} is set to <code>true</code>.
      *
      * @parameter expression="${maven.compiler.meminitial}"
+     * @since 2.0.1
      */
     private String meminitial;
 
     /**
      * Sets the maximum size, in megabytes, of the memory allocation pool, ex. "128", "128m"
-     * if fork is set to true.
+     * if {@link #fork} is set to <code>true</code>.
      *
      * @parameter expression="${maven.compiler.maxmem}"
+     * @since 2.0.1
      */
     private String maxmem;
 
     /**
-     * Sets the executable of the compiler to use when fork is true.
+     * Sets the executable of the compiler to use when {@link #fork} is <code>true</code>.
      *
      * @parameter expression="${maven.compiler.executable}"
      */
@@ -178,20 +187,60 @@ public abstract class AbstractCompilerMojo
 
     /**
      * <p>
-     * Sets the arguments to be passed to the compiler (prepending a dash) if fork is set to true.
-     * </p>
-     * <p>
-     * This is because the list of valid arguments passed to a Java compiler
-     * varies based on the compiler version.
+     * Sets whether annotation processing is performed or not. Only applies to JDK 1.6+
+     * If not set, both compilation and annotation processing are performed at the same time.
      * </p>
+     * <p>Allowed values are:</p>
+     * <ul>
+     * <li><code>none</code> - no annotation processing is performed.</li>
+     * <li><code>only</code> - only annotation processing is done, no compilation.</li>
+     * </ul>
      *
      * @parameter
+     * @since 2.2
      */
-    private Map compilerArguments;
+    private String proc;
 
     /**
      * <p>
-     * Sets the unformatted argument string to be passed to the compiler if fork is set to true.
+     * Names of annotation processors to run. Only applies to JDK 1.6+
+     * If not set, the default annotation processors discovery process applies.
+     * </p>
+     *
+     * @parameter
+     * @since 2.2
+     */
+    private String[] annotationProcessors;
+
+    /**
+     * <p>
+     * Sets the arguments to be passed to the compiler (prepending a dash) if {@link #fork} is set to <code>true</code>.
+     * </p>
+     * <p>
+     * This is because the list of valid arguments passed to a Java compiler
+     * varies based on the compiler version.
+     * </p>
+     * <p>
+     * To pass <code>-Xmaxerrs 1000 -Xlint -Xlint:-path -Averbose=true</code> you should include the following:
+     * </p>
+     * <pre>
+     * &lt;compilerArguments&gt;
+     *   &lt;Xmaxerrs&gt;1000&lt;/Xmaxerrs&gt;
+     *   &lt;Xlint/&gt;
+     *   &lt;Xlint:-path/&gt;
+     *   &lt;Averbose&gt;true&lt;/Averbose&gt;
+     * &lt;/compilerArguments&gt;
+     * </pre>
+     *
+     * @parameter
+     * @since 2.0.1
+     */
+    protected Map<String, String> compilerArguments;
+
+    /**
+     * <p>
+     * Sets the unformatted single argument string to be passed to the compiler if {@link #fork} is set to <code>true</code>.
+     * To pass multiple arguments such as <code>-Xmaxerrs 1000</code> (which are actually two arguments) you have to use {@link #compilerArguments}.
      * </p>
      * <p>
      * This is because the list of valid arguments passed to a Java compiler
@@ -200,7 +249,7 @@ public abstract class AbstractCompilerMojo
      *
      * @parameter
      */
-    private String compilerArgument;
+    protected String compilerArgument;
 
     /**
      * Sets the name of the output file when compiling a set of
@@ -210,6 +259,22 @@ public abstract class AbstractCompilerMojo
      */
     private String outputFileName;
 
+    /**
+     * Keyword list to be appended to the <code>-g</code> command-line switch. Legal values are none or a 
+     * comma-separated list of the following keywords: <code>lines</code>, <code>vars</code>, and <code>source</code>.
+     * If debug level is not specified, by default, nothing will be appended to <code>-g</code>.
+     * If debug is not turned on, this attribute will be ignored.
+     *
+     * @parameter expression="${maven.compiler.debuglevel}"
+     * @since 2.1
+     */
+    private String debuglevel;
+
+    /**
+     * @component
+     */
+    private ToolchainManager toolchainManager;
+
     // ----------------------------------------------------------------------
     // Read-only parameters
     // ----------------------------------------------------------------------
@@ -217,7 +282,7 @@ public abstract class AbstractCompilerMojo
     /**
      * The directory to run the compiler from if fork is true.
      *
-     * @parameter expression="${basedir}"
+     * @parameter default-value="${basedir}"
      * @required
      * @readonly
      */
@@ -226,7 +291,7 @@ public abstract class AbstractCompilerMojo
     /**
      * The target directory of the compiler if fork is true.
      *
-     * @parameter expression="${project.build.directory}"
+     * @parameter default-value="${project.build.directory}"
      * @required
      * @readonly
      */
@@ -239,16 +304,56 @@ public abstract class AbstractCompilerMojo
      */
     private CompilerManager compilerManager;
 
+    /**
+     * The current build session instance. This is used for toolchain manager API calls.
+     *
+     * @parameter default-value="${session}"
+     * @required
+     * @readonly
+     */
+    private MavenSession session;
+
+    /**
+     * Strategy to re use javacc class created:
+     * <ul>
+     *   <li><code>reuseCreated</code> (default): will reuse already created but in case of multi-threaded builds,
+     *   each thread will have its own instance</li>
+     *   <li><code>reuseSame</code>: the same Javacc class will be used for each compilation even for multi-threaded build</li>
+     *   <li><code>alwaysNew</code>: a new Javacc class will be created for each compilation</li>
+     * </ul>
+     * Note this parameter value depends on the os/jdk you are using, but the default value should work on most of env.
+     *
+     * @parameter default-value="${reuseCreated}" expression="${maven.compiler.compilerReuseStrategy}"
+     * @since 2.5
+     */
+    private String compilerReuseStrategy = "reuseCreated";
+
+    /**
+     * @parameter default-value="${false}" expression="${maven.compiler.skipMultiThreadWarning}"
+     * @since 2.5
+     */
+    private boolean skipMultiThreadWarning;
+
     protected abstract SourceInclusionScanner getSourceInclusionScanner( int staleMillis );
 
     protected abstract SourceInclusionScanner getSourceInclusionScanner( String inputFileEnding );
 
-    protected abstract List getClasspathElements();
+    protected abstract List<String> getClasspathElements();
 
-    protected abstract List getCompileSourceRoots();
+    protected abstract List<String> getCompileSourceRoots();
 
     protected abstract File getOutputDirectory();
 
+    protected abstract String getSource();
+
+    protected abstract String getTarget();
+
+    protected abstract String getCompilerArgument();
+
+    protected abstract Map<String, String> getCompilerArguments();
+
+    protected abstract File getGeneratedSourcesDirectory();
+
     public void execute()
         throws MojoExecutionException, CompilationFailureException
     {
@@ -271,11 +376,28 @@ public abstract class AbstractCompilerMojo
             throw new MojoExecutionException( "No such compiler '" + e.getCompilerId() + "'." );
         }
 
+        //-----------toolchains start here ----------------------------------
+        //use the compilerId as identifier for toolchains as well.
+        Toolchain tc = getToolchain();
+        if ( tc != null )
+        {
+            getLog().info( "Toolchain in compiler-plugin: " + tc );
+            if ( executable != null )
+            {
+                getLog().warn( "Toolchains are ignored, 'executable' parameter is set to " + executable );
+            }
+            else
+            {
+                fork = true;
+                //TODO somehow shaky dependency between compilerId and tool executable.
+                executable = tc.findTool( compilerId );
+            }
+        }
         // ----------------------------------------------------------------------
         //
         // ----------------------------------------------------------------------
 
-        List compileSourceRoots = removeEmptyCompileSourceRoots( getCompileSourceRoots() );
+        List<String> compileSourceRoots = removeEmptyCompileSourceRoots( getCompileSourceRoots() );
 
         if ( compileSourceRoots.isEmpty() )
         {
@@ -307,38 +429,70 @@ public abstract class AbstractCompilerMojo
 
         compilerConfiguration.setDebug( debug );
 
+        if ( debug && StringUtils.isNotEmpty( debuglevel ) )
+        {
+            String[] split = StringUtils.split( debuglevel, "," );
+            for ( int i = 0; i < split.length; i++ )
+            {
+                if ( !( split[i].equalsIgnoreCase( "none" ) || split[i].equalsIgnoreCase( "lines" )
+                    || split[i].equalsIgnoreCase( "vars" ) || split[i].equalsIgnoreCase( "source" ) ) )
+                {
+                    throw new IllegalArgumentException( "The specified debug level: '" + split[i] + "' is unsupported. "
+                                                            + "Legal values are 'none', 'lines', 'vars', and 'source'." );
+                }
+            }
+            compilerConfiguration.setDebugLevel( debuglevel );
+        }
+
         compilerConfiguration.setVerbose( verbose );
 
         compilerConfiguration.setShowWarnings( showWarnings );
 
         compilerConfiguration.setShowDeprecation( showDeprecation );
 
-        compilerConfiguration.setSourceVersion( source );
+        compilerConfiguration.setSourceVersion( getSource() );
 
-        compilerConfiguration.setTargetVersion( target );
+        compilerConfiguration.setTargetVersion( getTarget() );
+
+        compilerConfiguration.setProc( proc );
+
+        compilerConfiguration.setGeneratedSourcesDirectory( getGeneratedSourcesDirectory() );
+
+        compilerConfiguration.setAnnotationProcessors( annotationProcessors );
 
         compilerConfiguration.setSourceEncoding( encoding );
 
-        if ( ( compilerArguments != null ) || ( compilerArgument != null ) )
+        Map<String, String> effectiveCompilerArguments = getCompilerArguments();
+
+        String effectiveCompilerArgument = getCompilerArgument();
+
+        if ( ( effectiveCompilerArguments != null ) || ( effectiveCompilerArgument != null ) )
         {
-            LinkedHashMap cplrArgsCopy = new LinkedHashMap();
-            if ( compilerArguments != null )
+            LinkedHashMap<String, String> cplrArgsCopy = new LinkedHashMap<String, String>();
+            if ( effectiveCompilerArguments != null )
             {
-                for ( Iterator i = compilerArguments.entrySet().iterator(); i.hasNext(); )
+                for ( Map.Entry<String, String> me : effectiveCompilerArguments.entrySet() )
                 {
-                    Map.Entry me = (Map.Entry) i.next();
-                    String key = (String) me.getKey();
-                    String value = (String) me.getValue();
+                    String key = me.getKey();
+                    String value = me.getValue();
                     if ( !key.startsWith( "-" ) )
                     {
                         key = "-" + key;
                     }
-                    cplrArgsCopy.put( key, value );
+
+                    if ( key.startsWith( "-A" ) && StringUtils.isNotEmpty( value ) )
+                    {
+                        cplrArgsCopy.put( key + "=" + value, null );
+                    }
+                    else
+                    {
+                        cplrArgsCopy.put( key, value );
+                    }
                 }
             }
-            if ( !StringUtils.isEmpty( compilerArgument ) )
+            if ( !StringUtils.isEmpty( effectiveCompilerArgument ) )
             {
-                cplrArgsCopy.put( compilerArgument, null );
+                cplrArgsCopy.put( effectiveCompilerArgument, null );
             }
             compilerConfiguration.setCustomCompilerArguments( cplrArgsCopy );
         }
@@ -386,8 +540,37 @@ public abstract class AbstractCompilerMojo
 
         compilerConfiguration.setOutputFileName( outputFileName );
 
+        if ( CompilerConfiguration.CompilerReuseStrategy.AlwaysNew.getStrategy().equals( this.compilerReuseStrategy ) )
+        {
+            compilerConfiguration.setCompilerReuseStrategy( CompilerConfiguration.CompilerReuseStrategy.AlwaysNew );
+        }
+        else if ( CompilerConfiguration.CompilerReuseStrategy.ReuseSame.getStrategy().equals(
+            this.compilerReuseStrategy ) )
+        {
+            if ( getRequestThreadCount() > 1 )
+            {
+                if ( !skipMultiThreadWarning )
+                {
+                    StringBuilder sb = new StringBuilder(
+                        "You are in a multi-thread build and compilerReuseStrategy is set to reuseSame. This can cause issues in some environments (os/jdk)! Consider using reuseCreated strategy." );
+                    sb.append( System.getProperty( "line.separator" ) );
+                    sb.append(
+                        "If your env is fine with reuseSame, you can skip this warning with the configuration field skipMultiThreadWarning or -Dmaven.compiler.skipMultiThreadWarning=true" );
+                    getLog().warn( sb.toString() );
+                }
+            }
+            compilerConfiguration.setCompilerReuseStrategy( CompilerConfiguration.CompilerReuseStrategy.ReuseSame );
+        }
+        else
+        {
+
+            compilerConfiguration.setCompilerReuseStrategy( CompilerConfiguration.CompilerReuseStrategy.ReuseCreated );
+        }
+
+        getLog().debug( "CompilerReuseStrategy: " + compilerConfiguration.getCompilerReuseStrategy().getStrategy() );
+
         // TODO: have an option to always compile (without need to clean)
-        Set staleSources;
+        Set<File> staleSources;
 
         boolean canUpdateTarget;
 
@@ -398,15 +581,15 @@ public abstract class AbstractCompilerMojo
 
             canUpdateTarget = compiler.canUpdateTarget( compilerConfiguration );
 
-            if ( compiler.getCompilerOutputStyle().equals( CompilerOutputStyle.ONE_OUTPUT_FILE_FOR_ALL_INPUT_FILES ) &&
-                !canUpdateTarget )
+            if ( compiler.getCompilerOutputStyle().equals( CompilerOutputStyle.ONE_OUTPUT_FILE_FOR_ALL_INPUT_FILES )
+                && !canUpdateTarget )
             {
                 getLog().info( "RESCANNING!" );
                 // TODO: This second scan for source files is sub-optimal
                 String inputFileEnding = compiler.getInputFileEnding( compilerConfiguration );
 
-                Set sources = computeStaleSources( compilerConfiguration, compiler,
-                                                   getSourceInclusionScanner( inputFileEnding ) );
+                Set<File> sources = computeStaleSources( compilerConfiguration, compiler,
+                                                         getSourceInclusionScanner( inputFileEnding ) );
 
                 compilerConfiguration.setSourceFiles( sources );
             }
@@ -435,58 +618,60 @@ public abstract class AbstractCompilerMojo
         {
             getLog().debug( "Classpath:" );
 
-            for ( Iterator it = getClasspathElements().iterator(); it.hasNext(); )
+            for ( String s : getClasspathElements() )
             {
-                String s = (String) it.next();
-
                 getLog().debug( " " + s );
             }
 
             getLog().debug( "Source roots:" );
 
-            for ( Iterator it = getCompileSourceRoots().iterator(); it.hasNext(); )
+            for ( String root : getCompileSourceRoots() )
             {
-                String root = (String) it.next();
-
                 getLog().debug( " " + root );
             }
 
-            if ( fork )
+            try
             {
-                try
+                if ( fork )
                 {
                     if ( compilerConfiguration.getExecutable() != null )
                     {
                         getLog().debug( "Excutable: " );
                         getLog().debug( " " + compilerConfiguration.getExecutable() );
                     }
+                }
 
-                    String[] cl = compiler.createCommandLine( compilerConfiguration );
-                    if ( cl != null && cl.length > 0 )
-                    {
-                        StringBuffer sb = new StringBuffer();
-                        sb.append( cl[0] );
-                        for ( int i = 1; i < cl.length; i++ )
-                        {
-                            sb.append( " " );
-                            sb.append( cl[i] );
-                        }
-                        getLog().debug( "Command line options:" );
-                        getLog().debug( sb );
-                    }
-                }
-                catch ( CompilerException ce )
+                String[] cl = compiler.createCommandLine( compilerConfiguration );
+                if ( cl != null && cl.length > 0 )
                 {
-                    getLog().debug( ce );
+                    StringBuilder sb = new StringBuilder();
+                    sb.append( cl[0] );
+                    for ( int i = 1; i < cl.length; i++ )
+                    {
+                        sb.append( " " );
+                        sb.append( cl[i] );
+                    }
+                    getLog().debug( "Command line options:" );
+                    getLog().debug( sb );
                 }
             }
+            catch ( CompilerException ce )
+            {
+                getLog().debug( ce );
+            }
         }
 
         // ----------------------------------------------------------------------
         // Compile!
         // ----------------------------------------------------------------------
 
-        List messages;
+        if ( StringUtils.isEmpty( compilerConfiguration.getSourceEncoding() ) )
+        {
+            getLog().warn( "File encoding has not been set, using platform encoding " + ReaderFactory.FILE_ENCODING
+                               + ", i.e. build is platform dependent!" );
+        }
+
+        List<CompilerError> messages;
 
         try
         {
@@ -498,34 +683,82 @@ public abstract class AbstractCompilerMojo
             throw new MojoExecutionException( "Fatal error compiling", e );
         }
 
-        boolean compilationError = false;
-
-        for ( Iterator i = messages.iterator(); i.hasNext(); )
+        List<CompilerError> warnings = new ArrayList<CompilerError>();
+        List<CompilerError> errors = new ArrayList<CompilerError>();
+        if ( messages != null )
         {
-            CompilerError message = (CompilerError) i.next();
-
-            if ( message.isError() )
+            for ( CompilerError message : messages )
             {
-                compilationError = true;
-                break;
+                if ( message.isError() )
+                {
+                    errors.add( message );
+                }
+                else
+                {
+                    warnings.add( message );
+                }
             }
         }
 
-        if ( compilationError && failOnError )
+        if ( failOnError && !errors.isEmpty() )
         {
-            throw new CompilationFailureException( messages );
+            if ( !warnings.isEmpty() )
+            {
+                getLog().info( "-------------------------------------------------------------" );
+                getLog().warn( "COMPILATION WARNING : " );
+                getLog().info( "-------------------------------------------------------------" );
+                for ( CompilerError warning : warnings )
+                {
+                    getLog().warn( warning.toString() );
+                }
+                getLog().info( warnings.size() + ( ( warnings.size() > 1 ) ? " warnings " : " warning" ) );
+                getLog().info( "-------------------------------------------------------------" );
+            }
+
+            getLog().info( "-------------------------------------------------------------" );
+            getLog().error( "COMPILATION ERROR : " );
+            getLog().info( "-------------------------------------------------------------" );
+
+            for ( CompilerError error : errors )
+            {
+                getLog().error( error.toString() );
+            }
+            getLog().info( errors.size() + ( ( errors.size() > 1 ) ? " errors " : " error" ) );
+            getLog().info( "-------------------------------------------------------------" );
+
+            throw new CompilationFailureException( errors );
         }
         else
         {
-            for ( Iterator i = messages.iterator(); i.hasNext(); )
+            for ( CompilerError message : messages )
             {
-                CompilerError message = (CompilerError) i.next();
-
                 getLog().warn( message.toString() );
             }
         }
     }
 
+    /**
+     * try to get thread count if a Maven 3 build, using reflection as the plugin must not be maven3 api dependant
+     *
+     * @return number of thread for this build or 1 if not multi-thread build
+     */
+    protected int getRequestThreadCount()
+    {
+        try
+        {
+            Method getRequestMethod = this.session.getClass().getMethod( "getRequest" );
+            Object mavenExecutionRequest = getRequestMethod.invoke( this.session );
+            Method getThreadCountMethod = mavenExecutionRequest.getClass().getMethod( "getThreadCount" );
+            String threadCount = (String) getThreadCountMethod.invoke( mavenExecutionRequest );
+            return Integer.valueOf( threadCount );
+        }
+        catch ( Exception e )
+        {
+            getLog().debug( "unable to get threadCount for the current build: " + e.getMessage() );
+        }
+        return 1;
+    }
+
     private String getMemoryValue( String setting )
     {
         String value = null;
@@ -537,8 +770,8 @@ public abstract class AbstractCompilerMojo
         }
         else
         {
-            if ( ( isDigits( setting.substring( 0, setting.length() - 1 ) ) ) &&
-                ( setting.toLowerCase().endsWith( "m" ) ) )
+            if ( ( isDigits( setting.substring( 0, setting.length() - 1 ) ) ) && ( setting.toLowerCase().endsWith(
+                "m" ) ) )
             {
                 value = setting;
             }
@@ -546,6 +779,18 @@ public abstract class AbstractCompilerMojo
         return value;
     }
 
+    //TODO remove the part with ToolchainManager lookup once we depend on
+    //3.0.9 (have it as prerequisite). Define as regular component field then.
+    private Toolchain getToolchain()
+    {
+        Toolchain tc = null;
+        if ( toolchainManager != null )
+        {
+            tc = toolchainManager.getToolchainFromBuildContext( "jdk", session );
+        }
+        return tc;
+    }
+
     private boolean isDigits( String string )
     {
         for ( int i = 0; i < string.length(); i++ )
@@ -558,8 +803,8 @@ public abstract class AbstractCompilerMojo
         return true;
     }
 
-    private Set computeStaleSources( CompilerConfiguration compilerConfiguration, Compiler compiler,
-                                     SourceInclusionScanner scanner )
+    private Set<File> computeStaleSources( CompilerConfiguration compilerConfiguration, Compiler compiler,
+                                           SourceInclusionScanner scanner )
         throws MojoExecutionException, CompilerException
     {
         CompilerOutputStyle outputStyle = compiler.getCompilerOutputStyle();
@@ -570,15 +815,15 @@ public abstract class AbstractCompilerMojo
 
         if ( outputStyle == CompilerOutputStyle.ONE_OUTPUT_FILE_PER_INPUT_FILE )
         {
-            mapping = new SuffixMapping( compiler.getInputFileEnding( compilerConfiguration ), compiler
-                .getOutputFileEnding( compilerConfiguration ) );
+            mapping = new SuffixMapping( compiler.getInputFileEnding( compilerConfiguration ),
+                                         compiler.getOutputFileEnding( compilerConfiguration ) );
 
             outputDirectory = getOutputDirectory();
         }
         else if ( outputStyle == CompilerOutputStyle.ONE_OUTPUT_FILE_FOR_ALL_INPUT_FILES )
         {
-            mapping = new SingleTargetSourceMapping( compiler.getInputFileEnding( compilerConfiguration ), compiler
-                .getOutputFile( compilerConfiguration ) );
+            mapping = new SingleTargetSourceMapping( compiler.getInputFileEnding( compilerConfiguration ),
+                                                     compiler.getOutputFile( compilerConfiguration ) );
 
             outputDirectory = buildDirectory;
         }
@@ -589,12 +834,10 @@ public abstract class AbstractCompilerMojo
 
         scanner.addSourceMapping( mapping );
 
-        Set staleSources = new HashSet();
+        Set<File> staleSources = new HashSet<File>();
 
-        for ( Iterator it = getCompileSourceRoots().iterator(); it.hasNext(); )
+        for ( String sourceRoot : getCompileSourceRoots() )
         {
-            String sourceRoot = (String) it.next();
-
             File rootFile = new File( sourceRoot );
 
             if ( !rootFile.isDirectory() )
@@ -609,7 +852,7 @@ public abstract class AbstractCompilerMojo
             catch ( InclusionScanException e )
             {
                 throw new MojoExecutionException(
-                    "Error scanning source root: \'" + sourceRoot + "\' " + "for stale files to recompile.", e );
+                    "Error scanning source root: \'" + sourceRoot + "\' for stale files to recompile.", e );
             }
         }
 
@@ -620,15 +863,14 @@ public abstract class AbstractCompilerMojo
      * @todo also in ant plugin. This should be resolved at some point so that it does not need to
      * be calculated continuously - or should the plugins accept empty source roots as is?
      */
-    private static List removeEmptyCompileSourceRoots( List compileSourceRootsList )
+    private static List<String> removeEmptyCompileSourceRoots( List<String> compileSourceRootsList )
     {
-        List newCompileSourceRootsList = new ArrayList();
+        List<String> newCompileSourceRootsList = new ArrayList<String>();
         if ( compileSourceRootsList != null )
         {
             // copy as I may be modifying it
-            for ( Iterator i = compileSourceRootsList.iterator(); i.hasNext(); )
+            for ( String srcDir : compileSourceRootsList )
             {
-                String srcDir = (String) i.next();
                 if ( !newCompileSourceRootsList.contains( srcDir ) && new File( srcDir ).exists() )
                 {
                     newCompileSourceRootsList.add( srcDir );
